import { verifyCheckCode, verifyParsedCheckCode, generateCheckCode, generateLookupData, parseCheckCode, packDataArrayBuffer, parseDataArrayBuffer } from "dr-js/module/common/module/TimedLookup";
import { createCacheMap } from "dr-js/module/common/data/CacheMap";
import { fetchLikeRequest } from "dr-js/module/node/net";
import { toArrayBuffer } from "dr-js/module/node/data/Buffer";
import { readFileAsync, writeFileAsync, createPathPrefixLock } from "dr-js/module/node/file/function";
const DEFAULT_AUTH_KEY = "auth-check-code";
const AUTH_SKIP = "auth-skip";
const AUTH_FILE = "auth-file";
const AUTH_FILE_GROUP = "auth-file-group";
const loadLookupFile = async pathFile => parseDataArrayBuffer(toArrayBuffer(await readFileAsync(pathFile)));
const authFetchTimedLookup = async (url, config, timedLookupData, authKey) => {
  const response = await fetchLikeRequest(url, {
    ...config,
    headers: {
      [authKey]: generateCheckCode(timedLookupData),
      ...config.headers
    }
  });
  if (!response.ok) throw new Error(`[authFetch] status: ${response.status}`);
  return response;
};
const configureAuthSkip = async ({authKey = "auth-check-code", logger, log = logger && logger.add}) => {
  log && log("auth skip enabled");
  const generateAuthCheckCode = async () => "auth-skip";
  return {
    authMode: "auth-skip",
    authKey,
    authFetch: fetchLikeRequest,
    checkAuth: () => "auth-skip",
    generateAuthCheckCode,
    generateAuthHeader: generateAuthCheckCode
  };
};
const configureAuthFile = async ({timedLookupData, authFile, authFileGenTag, authFileGenSize, authFileGenTokenSize, authFileGenTimeGap, authKey = "auth-check-code", logger, log = logger && logger.add}) => {
  if (!timedLookupData) {
    timedLookupData = await loadLookupFile(authFile).catch(async error => {
      if (!authFileGenTag) {
        log && log("missing auth file", error);
        throw error;
      }
      log && log("generate new auth file");
      const timedLookupData = await generateLookupData({
        tag: authFileGenTag,
        size: authFileGenSize,
        tokenSize: authFileGenTokenSize,
        timeGap: authFileGenTimeGap
      });
      await ((pathFile, LookupData) => writeFileAsync(pathFile, Buffer.from(packDataArrayBuffer(LookupData))))(authFile, timedLookupData);
      return timedLookupData;
    });
    log && log("loaded auth file");
  }
  const generateAuthCheckCode = () => generateCheckCode(timedLookupData);
  return {
    authMode: "auth-file",
    authKey,
    authFetch: (url, option) => authFetchTimedLookup(url, option, timedLookupData, authKey),
    checkAuth: checkCode => {
      verifyCheckCode(timedLookupData, checkCode);
      return timedLookupData.tag;
    },
    generateAuthCheckCode,
    generateAuthHeader: generateAuthCheckCode
  };
};
const configureAuthFileGroup = async ({authFileGroupPath, authFileGroupDefaultTag, authFileGroupKeySuffix, getFileNameForTag = authFileGroupKeySuffix ? tag => `${tag}${authFileGroupKeySuffix}` : tag => `${tag}.key`, authCacheExpireTime = 3e5, authCacheMap = createCacheMap({
  valueSizeSumMax: 8388608,
  eventHub: null
}), authKey = "auth-check-code", logger, log = logger && logger.add}) => {
  const getPath = createPathPrefixLock(authFileGroupPath);
  const getTimedLookupData = tag => authCacheMap.get(tag) || loadLookupFile(getPath(getFileNameForTag(tag))).then(timedLookupData => {
    authCacheMap.set(tag, timedLookupData, timedLookupData.dataView.byteLength, Date.now() + authCacheExpireTime);
    log && log(`loaded auth file for tag: ${tag}`);
    return timedLookupData;
  }, error => {
    log && log(`no auth file for tag: ${tag}`);
    throw error;
  });
  const generateAuthCheckCode = async (tag = authFileGroupDefaultTag) => generateCheckCode(await getTimedLookupData(tag));
  return {
    authMode: "auth-file-group",
    authKey,
    authFetch: async (url, option, tag = authFileGroupDefaultTag) => authFetchTimedLookup(url, option, await getTimedLookupData(tag), authKey),
    checkAuth: async checkCode => {
      const parsedCheckCode = parseCheckCode(checkCode);
      const tag = parsedCheckCode[0];
      const timedLookupData = await getTimedLookupData(tag);
      verifyParsedCheckCode(timedLookupData, parsedCheckCode);
      return timedLookupData.tag;
    },
    generateAuthCheckCode,
    generateAuthHeader: generateAuthCheckCode
  };
};
export { DEFAULT_AUTH_KEY, AUTH_SKIP, AUTH_FILE, AUTH_FILE_GROUP, configureAuthSkip, configureAuthFile, configureAuthFileGroup };