import { join, resolve, relative } from "path";
import { spawn } from "child_process";
import { getTimestamp } from "dr-js/module/common/time";
import { string, boolean, basicArray, basicObject } from "dr-js/module/common/verify";
import { statAsync, openAsync, truncateAsync, readFileAsync, writeFileAsync, createReadStream, createPathPrefixLock, toPosixPath } from "dr-js/module/node/file/function";
import { createDirectory } from "dr-js/module/node/file/File";
import { getDirectorySubInfoList } from "dr-js/module/node/file/Directory";
import { modify } from "dr-js/module/node/file/Modify";
import { getProcessListAsync, toProcessPidMap, findProcessPidMapInfo, toProcessTree, findProcessTreeInfo, killProcessTreeInfoAsync, getAllProcessStatusAsync, describeAllProcessStatusAsync, isPidExist } from "dr-js/module/node/system/ProcessStatus";
const TASK_ACTION_TYPE = {
  TASK_CONFIG_SET: "task-config:set",
  TASK_CONFIG_GET: "task-config:get",
  TASK_START: "task:start",
  TASK_STOP: "task:stop",
  TASK_DELETE: "task:delete",
  TASK_LIST: "task:list",
  TASK_LOG_GET: "task-log:get",
  TASK_LOG_GET_TAIL: "task-log:get-tail",
  TASK_LOG_RESET: "task-log:reset",
  PROCESS_STATUS: "process-status"
};
const REGEXP_TASK_KEY = /^\w[\w-.|]*$/;
const verifyString = (value, valueName) => {
  if (!value) throw new Error(`missing ${valueName}`);
  string(value, `invalid ${valueName}`);
  if (!value.length) throw new Error(`expect value in ${valueName}`);
};
const existTaskProcessAsync = async ({status}) => status && status.processInfo && isPidExist(status.processInfo.pid) && findProcessPidMapInfo(status.processInfo, toProcessPidMap(await getProcessListAsync()));
const onLoadConfigError = error => null;
const createTaskAction = rootPath => {
  const getPath = createPathPrefixLock(rootPath);
  const getConfigPath = key => getPath(join(key, "task.config.json"));
  const getLogPath = key => getPath(join(key, "task.log"));
  const saveConfig = config => writeFileAsync(getConfigPath(config.key), JSON.stringify(config));
  const loadConfig = async key => JSON.parse(await readFileAsync(getConfigPath(key)));
  const TASK_ACTION_MAP = {
    "task-config:set": async payload => {
      let config = (({key, task, info = {}}, getPath) => {
        verifyString(key, "key");
        if (!REGEXP_TASK_KEY.test(key)) throw new Error(`invalid key, get: ${key}, expect: ${REGEXP_TASK_KEY}`);
        basicObject(task, "invalid task");
        const taskPath = getPath(key);
        const {command, argList = [], cwd = taskPath, env = {}, shell = !0, resetLog = !0} = task;
        verifyString(command, "task.command");
        basicArray(argList, "invalid task.argList");
        basicObject(env, "invalid task.env");
        boolean(shell, "invalid task.shell");
        boolean(resetLog, "invalid task.resetLog");
        task = {
          command,
          argList,
          cwd: toPosixPath(relative(taskPath, resolve(taskPath, cwd))),
          env,
          shell,
          resetLog
        };
        basicObject(info, "invalid info");
        const {note} = info;
        const timeCreate = getTimestamp();
        return {
          key,
          task,
          info: info = {
            note,
            timeCreate,
            timeUpdate: timeCreate
          },
          status: null
        };
      })(payload, getPath);
      const existConfig = await loadConfig(config.key).catch(onLoadConfigError);
      if (existConfig && await existTaskProcessAsync(existConfig)) throw new Error("config task running");
      existConfig && (config = ((prevConfig, {key, task, info}) => ({
        key,
        task,
        info: {
          ...info,
          timeCreate: prevConfig.info.timeCreate
        },
        status: null
      }))(existConfig, config));
      await createDirectory(getPath(config.key));
      await saveConfig(config);
    },
    "task-config:get": ({key}) => loadConfig(key).catch(onLoadConfigError),
    "task:start": async ({key}) => {
      const config = await loadConfig(key);
      if (await existTaskProcessAsync(config)) throw new Error("task exist");
      config.task.resetLog && await TASK_ACTION_MAP["task-log:reset"]({
        key
      });
      const {processInfo} = await (async ({command, argList, cwd, env, shell = !0}, logFile) => {
        const subProcess = spawn(command, argList, {
          stdio: [ "ignore", await openAsync(logFile, "a"), await openAsync(logFile, "a") ],
          cwd,
          env,
          shell,
          detached: !0,
          windowsHide: !0
        });
        subProcess.on("error", error => {
          console.warn("[ERROR][startDetachedProcess] config:", {
            command,
            argList,
            cwd,
            env,
            shell
          }, "error:", error);
        });
        subProcess.unref();
        {
          const processList = await getProcessListAsync();
          const subProcessInfo = toProcessPidMap(processList)[subProcess.pid];
          if (!subProcessInfo || subProcessInfo.ppid !== process.pid) throw new Error(`sub process info not found, expect pid: ${subProcess.pid}, ppid: ${process.pid}`);
          const {pid, command, subTree} = findProcessTreeInfo(subProcessInfo, toProcessTree(processList));
          return {
            process: subProcess,
            processInfo: {
              pid,
              command,
              subTree
            }
          };
        }
      })(config.task, getLogPath(key));
      await saveConfig({
        ...config,
        status: {
          processInfo,
          timeStart: getTimestamp()
        }
      });
    },
    "task:stop": async ({key}) => {
      const config = await loadConfig(key);
      config.status && config.status.processInfo && await killProcessTreeInfoAsync(config.status.processInfo);
      await saveConfig({
        ...config,
        status: null
      });
    },
    "task:delete": async ({key}) => {
      const config = await loadConfig(key);
      if (await existTaskProcessAsync(config)) throw new Error(`task process running at: ${config.status.processInfo.pid}`);
      await modify.delete(getPath(key));
    },
    "task:list": async () => {
      const configList = [];
      for (const {name, stat} of await getDirectorySubInfoList(rootPath).catch(() => [])) {
        const config = stat.isDirectory() && await loadConfig(name).catch(onLoadConfigError);
        config && configList.push(config);
      }
      return {
        configList
      };
    },
    "task-log:get": async ({key}) => ({
      stream: createReadStream(getLogPath(key))
    }),
    "task-log:get-tail": async ({key, tailSize = 4096}) => {
      const logPath = getLogPath(key);
      const {size} = await statAsync(logPath);
      return {
        stream: createReadStream(logPath, {
          start: Math.max(size - tailSize, 0)
        })
      };
    },
    "task-log:reset": async ({key}) => truncateAsync(getLogPath(key)),
    "process-status": async ({outputMode = "tree", isHumanReadableOutput = !0}) => ({
      processStatus: await (isHumanReadableOutput ? describeAllProcessStatusAsync : getAllProcessStatusAsync)(outputMode)
    })
  };
  return async (type, payload) => ({
    type,
    ...await TASK_ACTION_MAP[type](payload)
  });
};
export { TASK_ACTION_TYPE, createTaskAction };